#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.illustration.interfaces import IIllustrationParagraph, ILLUSTRATION_PARAGRAPH_TYPE, \
    ILLUSTRATION_PARAGRAPH_NAME
from pyams_content.component.paragraph.interfaces import IParagraphFactory
from pyams_content.features.checker.interfaces import IContentChecker
from pyams_i18n.interfaces import II18n

# import packages
from pyams_content.component.illustration import Illustration as BaseIllustration, IllustrationContentChecker
from pyams_content.component.paragraph import BaseParagraph, BaseParagraphFactory
from pyams_utils.adapter import adapter_config
from pyams_utils.registry import utility_config
from pyams_utils.request import check_request
from zope.interface import implementer


@implementer(IIllustrationParagraph)
class Illustration(BaseIllustration, BaseParagraph):
    """Illustration class"""

    icon_class = 'fa-file-image-o'
    icon_hint = ILLUSTRATION_PARAGRAPH_NAME


@utility_config(name=ILLUSTRATION_PARAGRAPH_TYPE, provides=IParagraphFactory)
class IllustrationFactory(BaseParagraphFactory):
    """Illustration paragraph factory"""

    name = ILLUSTRATION_PARAGRAPH_NAME
    content_type = Illustration


@adapter_config(context=IIllustrationParagraph, provides=IContentChecker)
class IllustrationParagraphContentChecker(IllustrationContentChecker):
    """Illustration paragraph content checker"""

    @property
    def label(self):
        request = check_request()
        translate = request.localizer.translate
        return II18n(self.context).query_attribute('title', request) or \
            '({0})'.format(translate(self.context.icon_hint).lower())
